@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2006-2018 sqlmap developers (http://sqlmap.org/)
+Copyright (c) 2006-2020 sqlmap developers (http://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
@@ -25,15 +25,19 @@ except ImportError:
 try:
     import MySQLdb  # used by SQLAlchemy in case of MySQL
     warnings.filterwarnings("error", category=MySQLdb.Warning)
-except ImportError:
+except (ImportError, AttributeError):
     pass
 
 from lib.core.data import conf
 from lib.core.data import logger
 from lib.core.exception import SqlmapConnectionException
 from lib.core.exception import SqlmapFilePathException
+from lib.core.exception import SqlmapMissingDependence
 from plugins.generic.connector import Connector as GenericConnector
 
+def getSafeExString(ex, encoding=None):  # Cross-referenced function
+    raise NotImplementedError
+
 class SQLAlchemy(GenericConnector):
     def __init__(self, dialect=None):
         GenericConnector.__init__(self)
@@ -57,7 +61,7 @@ class SQLAlchemy(GenericConnector):
                 if self.dialect == "sqlite":
                     engine = _sqlalchemy.create_engine(conf.direct, connect_args={"check_same_thread": False})
                 elif self.dialect == "oracle":
-                    engine = _sqlalchemy.create_engine(conf.direct, connect_args={"allow_twophase": False})
+                    engine = _sqlalchemy.create_engine(conf.direct)
                 else:
                     engine = _sqlalchemy.create_engine(conf.direct, connect_args={})
 
@@ -72,13 +76,16 @@ class SQLAlchemy(GenericConnector):
                         pass
                 elif "invalid literal for int() with base 10: '0b" in traceback.format_exc():
                     raise SqlmapConnectionException("SQLAlchemy connection issue ('https://bitbucket.org/zzzeek/sqlalchemy/issues/3975')")
-                raise
+                else:
+                    pass
             except SqlmapFilePathException:
                 raise
-            except Exception, msg:
-                raise SqlmapConnectionException("SQLAlchemy connection issue ('%s')" % msg[0])
+            except Exception as ex:
+                raise SqlmapConnectionException("SQLAlchemy connection issue ('%s')" % getSafeExString(ex))
 
             self.printConnected()
+        else:
+            raise SqlmapMissingDependence("SQLAlchemy not available")
 
     def fetchall(self):
         try:
@@ -86,17 +93,17 @@ class SQLAlchemy(GenericConnector):
             for row in self.cursor.fetchall():
                 retVal.append(tuple(row))
             return retVal
-        except _sqlalchemy.exc.ProgrammingError, msg:
-            logger.log(logging.WARN if conf.dbmsHandler else logging.DEBUG, "(remote) %s" % msg.message if hasattr(msg, "message") else msg)
+        except _sqlalchemy.exc.ProgrammingError as ex:
+            logger.log(logging.WARN if conf.dbmsHandler else logging.DEBUG, "(remote) %s" % getSafeExString(ex))
             return None
 
     def execute(self, query):
         try:
             self.cursor = self.connector.execute(query)
-        except (_sqlalchemy.exc.OperationalError, _sqlalchemy.exc.ProgrammingError), msg:
-            logger.log(logging.WARN if conf.dbmsHandler else logging.DEBUG, "(remote) %s" % msg.message if hasattr(msg, "message") else msg)
-        except _sqlalchemy.exc.InternalError, msg:
-            raise SqlmapConnectionException(msg[1])
+        except (_sqlalchemy.exc.OperationalError, _sqlalchemy.exc.ProgrammingError) as ex:
+            logger.log(logging.WARN if conf.dbmsHandler else logging.DEBUG, "(remote) %s" % getSafeExString(ex))
+        except _sqlalchemy.exc.InternalError as ex:
+            raise SqlmapConnectionException(getSafeExString(ex))
 
     def select(self, query):
         self.execute(query)
