@@ -1,15 +1,31 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2006-2016 sqlmap developers (http://sqlmap.org/)
-See the file 'doc/COPYING' for copying permission
+Copyright (c) 2006-2020 sqlmap developers (http://sqlmap.org/)
+See the file 'LICENSE' for copying permission
 """
 
+import numbers
+
 class xrange(object):
     """
     Advanced (re)implementation of xrange (supports slice/copy/etc.)
     Reference: http://code.activestate.com/recipes/521885-a-pythonic-implementation-of-xrange/
 
+    >>> list(xrange(1, 9)) == list(range(1, 9))
+    True
+    >>> list(xrange(8, 0, -16)) == list(range(8, 0, -16))
+    True
+    >>> list(xrange(0, 8, 16)) == list(range(0, 8, 16))
+    True
+    >>> list(xrange(0, 4, 5)) == list(range(0, 4, 5))
+    True
+    >>> list(xrange(4, 0, 3)) == list(range(4, 0, 3))
+    True
+    >>> list(xrange(0, -3)) == list(range(0, -3))
+    True
+    >>> list(xrange(0, 7, 2)) == list(range(0, 7, 2))
+    True
     >>> foobar = xrange(1, 10)
     >>> 7 in foobar
     True
@@ -48,19 +64,14 @@ class xrange(object):
     def __hash__(self):
         return hash(self._slice)
 
-    def __cmp__(self, other):
-        return (cmp(type(self), type(other)) or
-                cmp(self._slice, other._slice))
-
     def __repr__(self):
-        return '%s(%r, %r, %r)' % (type(self).__name__,
-                                   self.start, self.stop, self.step)
+        return '%s(%r, %r, %r)' % (type(self).__name__, self.start, self.stop, self.step)
 
     def __len__(self):
         return self._len()
 
     def _len(self):
-        return max(0, int((self.stop - self.start) / self.step))
+        return max(0, 1 + int((self.stop - 1 - self.start) // self.step))
 
     def __contains__(self, value):
         return (self.start <= value < self.stop) and (value - self.start) % self.step == 0
@@ -69,8 +80,8 @@ class xrange(object):
         if isinstance(index, slice):
             start, stop, step = index.indices(self._len())
             return xrange(self._index(start),
-                          self._index(stop), step*self.step)
-        elif isinstance(index, (int, long)):
+                          self._index(stop), step * self.step)
+        elif isinstance(index, numbers.Integral):
             if index < 0:
                 fixed_index = index + self._len()
             else:
@@ -85,3 +96,9 @@ class xrange(object):
 
     def _index(self, i):
         return self.start + self.step * i
+
+    def index(self, i):
+        if self.start <= i < self.stop:
+            return i - self.start
+        else:
+            raise ValueError("%d is not in list" % i)
