@@ -1,17 +1,18 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2006-2017 sqlmap developers (http://sqlmap.org/)
-See the file 'doc/COPYING' for copying permission
+Copyright (c) 2006-2020 sqlmap developers (http://sqlmap.org/)
+See the file 'LICENSE' for copying permission
 """
 
 try:
     import pyodbc
-except ImportError:
+except:
     pass
 
 import logging
 
+from lib.core.common import getSafeExString
 from lib.core.data import conf
 from lib.core.data import logger
 from lib.core.exception import SqlmapConnectionException
@@ -28,9 +29,6 @@ class Connector(GenericConnector):
     License: MIT
     """
 
-    def __init__(self):
-        GenericConnector.__init__(self)
-
     def connect(self):
         if not IS_WIN:
             errMsg = "currently, direct connection to Microsoft Access database(s) "
@@ -42,8 +40,8 @@ class Connector(GenericConnector):
 
         try:
             self.connector = pyodbc.connect('Driver={Microsoft Access Driver (*.mdb)};Dbq=%s;Uid=Admin;Pwd=;' % self.db)
-        except (pyodbc.Error, pyodbc.OperationalError), msg:
-            raise SqlmapConnectionException(msg[1])
+        except (pyodbc.Error, pyodbc.OperationalError) as ex:
+            raise SqlmapConnectionException(getSafeExString(ex))
 
         self.initCursor()
         self.printConnected()
@@ -51,17 +49,17 @@ class Connector(GenericConnector):
     def fetchall(self):
         try:
             return self.cursor.fetchall()
-        except pyodbc.ProgrammingError, msg:
-            logger.log(logging.WARN if conf.dbmsHandler else logging.DEBUG, "(remote) %s" % msg[1])
+        except pyodbc.ProgrammingError as ex:
+            logger.log(logging.WARN if conf.dbmsHandler else logging.DEBUG, "(remote) %s" % getSafeExString(ex))
             return None
 
     def execute(self, query):
         try:
             self.cursor.execute(query)
-        except (pyodbc.OperationalError, pyodbc.ProgrammingError), msg:
-            logger.log(logging.WARN if conf.dbmsHandler else logging.DEBUG, "(remote) %s" % msg[1])
-        except pyodbc.Error, msg:
-            raise SqlmapConnectionException(msg[1])
+        except (pyodbc.OperationalError, pyodbc.ProgrammingError) as ex:
+            logger.log(logging.WARN if conf.dbmsHandler else logging.DEBUG, "(remote) %s" % getSafeExString(ex))
+        except pyodbc.Error as ex:
+            raise SqlmapConnectionException(getSafeExString(ex))
 
         self.connector.commit()
 
